#ifndef GENMANIP_STRUCT_H
#define GENMANIP_STRUCT_H

#include "gbGENOM.h"

/* maximal size of trajectories */
// Used to define the size of the poster position for TrackQ:
// 100 tics/seconde * 1 minute
#ifndef GENMANIP_POSITION_TRACK_MAX_SIZE
#define GENMANIP_POSITION_TRACK_MAX_SIZE 100*60*1
#endif

#ifndef GENMANIP_SEGMENT_TRACK_MAX_SIZE
#define GENMANIP_SEGMENT_TRACK_MAX_SIZE 200
#endif

#ifndef GENMANIP_MAX_CONFIG_SIZE
#define GENMANIP_MAX_CONFIG_SIZE	100
#endif

typedef struct GENMANIP_STRING1024 {
  char str[1024];
} GENMANIP_STRING1024;

typedef struct GENMANIP_STRING2048 {
  char str[2048];
} GENMANIP_STRING2048;

typedef enum GENMANIP_ON_OFF_ENUM {
  GENMANIP_OFF = 0,
  GENMANIP_ON = 1
} GENMANIP_ON_OFF_ENUM;

typedef enum GENMANIP_COLLISION_STATE_ENUM {
  GENMANIP_COLLISION_FREE = 0,
  GENMANIP_COLLISION_COLLISION
} GENMANIP_COLLISION_STATE_ENUM;

typedef struct GENMANIP_COLLISION_STATE_STR {
  GENMANIP_ON_OFF_ENUM mode;
  GENMANIP_COLLISION_STATE_ENUM state;
} GENMANIP_COLLISION_STATE_STR;

typedef struct GENMANIP_POSITION_TRACK_STR {
  int nb_points;
  int flag; /* position to continue tracking */
  Gb_q6 errorMax;
  int lp[GENMANIP_POSITION_TRACK_MAX_SIZE];
  Gb_q6 positions[GENMANIP_POSITION_TRACK_MAX_SIZE];
} GENMANIP_POSITION_TRACK_STR;

typedef struct GENMANIP_POSITION7_TRACK_STR {
  int   nb_points;
  int   track_position_index;
  int   flag; /* position to continue tracking */
  Gb_q7 errorMax;
  int   lp[GENMANIP_POSITION_TRACK_MAX_SIZE];
  Gb_q7 positions[GENMANIP_POSITION_TRACK_MAX_SIZE];
} GENMANIP_POSITION7_TRACK_STR;

typedef struct GENMANIP_POSITION_SAHAND_FINGER_STR {
  double base;
  double phaProx;
  double phaMid;
  double phaDist;
} GENMANIP_POSITION_SAHAND_FINGER_STR;

typedef struct GENMANIP_POSITION_SAHAND_STR {
  double thumbRotation;
  GENMANIP_POSITION_SAHAND_FINGER_STR thumb;
  GENMANIP_POSITION_SAHAND_FINGER_STR fore;
  GENMANIP_POSITION_SAHAND_FINGER_STR middle;
  GENMANIP_POSITION_SAHAND_FINGER_STR ring;
} GENMANIP_POSITION_SAHAND_STR;

typedef struct GENMANIP_POSITION_ARM_SAHAND_TRACK_STR {
  int   nb_points;
  int   track_position_index;
  int   flag; /* position to continue tracking */
  Gb_q7 errorMax;
  int   lp[GENMANIP_POSITION_TRACK_MAX_SIZE];
  Gb_q7 armRight[GENMANIP_POSITION_TRACK_MAX_SIZE];
  GENMANIP_POSITION_SAHAND_STR handRight[GENMANIP_POSITION_TRACK_MAX_SIZE];
}  GENMANIP_POSITION_ARM_SAHAND_TRACK_STR ;

typedef struct GENMANIP_SAHAND_STR {
  int cpt;
  GENMANIP_POSITION_SAHAND_STR handRight;
  GENMANIP_POSITION_SAHAND_STR handLeft;
} GENMANIP_SAHAND_STR;

/*
 * The two following structures are the new way 
 * to communicate between MHP and LWR
 */

typedef struct GENMANIP_SEGMENT7_STR {
  int nbAxis;
  double lp;
  double time;
  double ic_a[7];
  double ic_v[7];
  double ic_x[7];
  double jerk[7];
}GENMANIP_SEGMENT7_STR;

typedef struct GENMANIP_TRAJ7_TRACK_STR {
  int nb_segments;
  GENMANIP_SEGMENT7_STR seg[GENMANIP_SEGMENT_TRACK_MAX_SIZE];
} GENMANIP_TRAJ7_TRACK_STR ;

typedef struct GENMANIP_POSITION_STR {
  int lp;
  Gb_q7 q;
} GENMANIP_POSITION_STR;



typedef struct GENMANIP_CONFIG {
  float q[GENMANIP_MAX_CONFIG_SIZE];
  unsigned int size;
} GENMANIP_CONFIG;

typedef struct GENMANIP_MOTION_STR {
  double q[GENMANIP_MAX_CONFIG_SIZE];
  double vq[GENMANIP_MAX_CONFIG_SIZE];
  double aq[GENMANIP_MAX_CONFIG_SIZE];
  int size;
} GENMANIP_MOTION_STR;

typedef struct GENMANIP_DOF {
  unsigned int index;
  float value;
} GENMANIP_DOF;


typedef struct GENMANIP_SET_POSITION_STR {
  GEN_STRING128 name;
  float x;
  float y;
  float z;
} GENMANIP_SET_POSITION_STR;

typedef struct GENMANIP_SET_ORIENTATION_STR {
  GEN_STRING128 name;
  float yaw;
  float pitch;
  float roll;
} GENMANIP_SET_ORIENTATION_STR;

typedef struct GENMANIP_SET_POSE_STR {
  GEN_STRING128 name;
  float x;
  float y;
  float z;
  float yaw;
  float pitch;
  float roll;
} GENMANIP_SET_POSE_STR;

typedef enum GENMANIP_FORCE_EVENT_ENUM {
  GENMANIP_NONE      = 0,
  GENMANIP_GRASP     = 1,
  GENMANIP_COLLISION = 2
} GENMANIP_FORCE_EVENT_ENUM;

typedef struct GENMANIP_FORCE_EVENT_STR {
  GENMANIP_FORCE_EVENT_ENUM event_type;
} GENMANIP_FORCE_EVENT_STR;

typedef struct GENMANIP_MOUSE3D_STR {
  double x;
  double y;
  double z;
  double Rx;
  double Ry;
  double Rz;
  int b0;
  int b1;
}  GENMANIP_MOUSE3D_STR;

#endif
